/**
 * Created by petry_000 on 25.03.2017.
 */
angular.module('Qfe7')
    .run(function (pouchDB) {
        /*pouchDB('agent').replicate.to("http://127.0.0.1/agent", {
            live: true,
            retry: true
         });
        pouchDB('agent').replicate.from("http://127.0.0.1/agent", {
            live: true,
            retry: true
         });*/
    })
    .service("AgentService", function ($q, pouchDB) {
        const is = this;
        let db = pouchDB('agent');
        is.load = function () {
            return $q((rs, rj) => {
                db.allDocs({include_docs: true}).then(res => {
                    rs(_.map(res.rows, d => d.doc));
                }, rj)
            })
        };

        is.create = function (al) {
            let db = pouchDB('agent');
            return db.bulkDocs(al);
        };

        is.reset = function () {
            let db = pouchDB('agent');
            return db.destroy();
        }
    })
    .controller("RootController", function ($scope, $state, VisDataSet, AgentService, Agent, Link) {
        $scope.err = undefined;
        function errorFn(err) {
            $scope.err = err;
        }

        $scope.settings = {
            self: true,
        };

        $scope.options = {
            interaction: {
                hover: true
            },
            "edges": {
                "smooth": {
                    "forceDirection": "none"
                }
            },
            "physics": {
                "barnesHut": {
                    "gravitationalConstant": -10000,
                    "centralGravity": 0.1,
                    "springLength": 400,
                    "springConstant": 0.05,
                    "damping": 0.4,
                    "avoidOverlap": 0
                },
                "minVelocity": 0.75
            }
        };

        $scope.data = {
            nodes: new VisDataSet(),
            edges: new VisDataSet(),

            description: undefined
        };

        function eventHandler(x) {
            return function (e) {
                x(e);
                $scope.$apply('data');
            }
        }

        $scope.events = {
            selectNode: eventHandler(function (x) {
                let node = $scope.data.nodes.get(x.nodes[0]);
                $scope.data.description = JSON.stringify(node.agent);
            }),
            deselectNode: eventHandler(function (x) {
                $scope.data.description = undefined;
            })
        };

        const shapes = {
            'male': 'triangleDown',
            'female': 'triangle',
            'organization': 'dot'
        };

        const sizes = {
            'child': {size: 15},
            'mature': {size: 30},
            'monster': {size: 100}
        };

        const links = {
            'life': {
                color: 'red',
                arrows: 'from'
            },
            'food': {
                color: 'green',
                arrows: 'to'
            },
            'eat': {
                color: 'green',
                arrows: 'to'
            }
        };

        function getValueOf(link) {
            switch (link.type) {
                case 'life':
                    return (link.value / 2500);
                case 'food':
                    return link.value;
                case 'eat':
                    return link.value;
                default:
                    return 1;
            }
        }

        let doNodes = $scope.doNodes = function (x) {
            $scope.data.nodes.clear();
            $scope.data.edges.clear();
            AgentService.load().then(al => {
                Array.from(al).forEach(a => {
                    let node = {
                        agent: a,
                        id: a._id,
                        label: a.name,
                        shape: shapes[a.type],
                        color: a.skin,
                        x: Math.round(Math.random() * 100) - 50,
                        y: Math.round(Math.random() * 100) - 50
                    };
                    _.extend(node, sizes[a.age]);
                    $scope.data.nodes.add(node);
                });
                Array.from(al).forEach(a => {
                    Array.from(a.links)
                        .filter(l => {
                            return $scope.settings.self || !(l.type == 'life' || l.type == 'eat');
                        }).forEach(l => {
                        let link = {
                            id: l._id,
                            from: l.from.id,
                            to: l.to.id,
                            value: getValueOf(l)
                        };
                        _.extend(link, links[l.type]);
                        $scope.data.edges.add(link)
                    })
                });
            }, errorFn)
        };

        doNodes();

        $scope.initAgents = function () {
            let m = Agent.is().male().build();
            Link.for(m).life().build();
            Link.for(m).eat().build();

            let f = Agent.is().female().build();
            Link.for(f).life().build();
            Link.for(f).eat().build();

            let cm = Agent.is().male().child().build();
            Link.for(cm).life().build();
            Link.for(cm).eat().build();

            let cf = Agent.is().female().child().build();
            Link.for(cf).life().build();
            Link.for(cf).eat().build();

            Link.for(m).food(0.5).to(cm).build();
            Link.for(m).food(0.5).to(cf).build();
            Link.for(f).food(0.5).to(cm).build();
            Link.for(f).food(0.5).to(cf).build();

            let o = Agent.is().organization().build();

            let l = Agent.is().organization().has('food').build();
            Link.for(l).food(2).to(m).build();
            Link.for(l).food(2).to(f).build();

            AgentService.create([m, f, cm, cf, o, l]).then(doNodes, errorFn);
        };

        $scope.resetAgents = function () {
            AgentService.reset();
            window.location.reload();
        };

    });